@@ -32,6 +32,9 @@
 #include "preferences.h"
 
 
+#define MAX_LINES 100
+
+
 Bookmarks *
 bookmarks_new (char *rc_filename)
 {
@@ -41,6 +44,7 @@ bookmarks_new (char *rc_filename)
 	bookmarks->list = NULL;
 	bookmarks->names = g_hash_table_new (g_str_hash, g_str_equal);
 	bookmarks->tips = g_hash_table_new (g_str_hash, g_str_equal);
+	bookmarks->max_lines = MAX_LINES;
 
 	if (rc_filename != NULL)
 		bookmarks->rc_filename = g_strdup (rc_filename);
@@ -94,9 +98,9 @@ bookmarks_free (Bookmarks *bookmarks)
 char *
 bookmarks_utils__get_menu_item_name (const char *path)
 {
-	char     *name;
 	char     *tmp_path;
 	gboolean  catalog_or_search;
+	char     *name;
 
 	tmp_path = g_strdup (pref_util_remove_prefix (path));
 
@@ -209,7 +213,8 @@ my_remove (GHashTable    *hash_table,
 void
 bookmarks_add (Bookmarks   *bookmarks,
 	       const char  *path,
-	       gboolean     avoid_duplicates)
+	       gboolean     avoid_duplicates,
+	       gboolean     append)
 {
 	g_return_if_fail (bookmarks != NULL);
 	g_return_if_fail (path != NULL);	
@@ -221,7 +226,12 @@ bookmarks_add (Bookmarks   *bookmarks,
 				return;
 	}
 	
-	bookmarks->list = g_list_append (bookmarks->list, g_strdup (path));
+	if (append)
+		bookmarks->list = g_list_append (bookmarks->list, 
+						 g_strdup (path));
+	else
+		bookmarks->list = g_list_prepend (bookmarks->list, 
+						  g_strdup (path));
 
 	my_insert (bookmarks->names, 
 		   path, 
@@ -233,24 +243,6 @@ bookmarks_add (Bookmarks   *bookmarks,
 }
 
 
-void
-bookmarks_add_with_prefix (Bookmarks   *bookmarks,
-			   const gchar *path,
-			   const gchar *prefix,
-			   gboolean     avoid_duplicates)
-{
-	gchar *full_path;
-
-	g_return_if_fail (bookmarks != NULL);
-	g_return_if_fail (path != NULL);	
-
-	full_path = g_strconcat (prefix, path, NULL);
-	
-	bookmarks_add (bookmarks, full_path, avoid_duplicates);
-	g_free (full_path);
-}
-
-
 static GList *
 get_link_from_path (GList      *list, 
 		    const char *path)
@@ -280,13 +272,36 @@ bookmarks_remove (Bookmarks  *bookmarks,
 
 	bookmarks->list = g_list_remove_link (bookmarks->list, link);
 	g_list_free (link);
+	g_free (link->data);
 
 	if (get_link_from_path (bookmarks->list, path) == NULL) {
 		my_remove (bookmarks->names, path);
 		my_remove (bookmarks->tips, path);
 	}
+}
 
-	g_free (link->data);
+
+void
+bookmarks_remove_all_instances (Bookmarks   *bookmarks,
+				const char  *path)
+{
+	GList *link;
+
+	g_return_if_fail (bookmarks != NULL);
+	g_return_if_fail (path != NULL);	
+
+	link = get_link_from_path (bookmarks->list, path);
+
+	while (link != NULL) {
+		bookmarks->list = g_list_remove_link (bookmarks->list, link);
+		g_free (link->data);
+		g_list_free (link);
+
+		link = get_link_from_path (bookmarks->list, path);
+	}
+
+	my_remove (bookmarks->names, path);
+	my_remove (bookmarks->tips, path);
 }
 
 
@@ -374,7 +389,6 @@ bookmarks_load_from_disk (Bookmarks *bookmarks)
 #undef MAX_LINE_LENGTH	
 
 
-#define MAX_LINES 100
 void
 bookmarks_write_to_disk (Bookmarks *bookmarks)
 {
@@ -403,7 +417,7 @@ bookmarks_write_to_disk (Bookmarks *bookmarks)
 	/* write the file list. */
 	lines = 0;
 	scan = bookmarks->list; 
-	while ((lines < MAX_LINES) && scan) {
+	while ((lines < bookmarks->max_lines) && scan) {
 		if (! fprintf (f, "\"%s\"\n", (gchar*) scan->data)) {
 			g_print ("ERROR saving to bookmark file\n");
 			fclose (f);
@@ -433,3 +447,18 @@ bookmarks_get_menu_tip (Bookmarks *bookmarks,
 {
 	return g_hash_table_lookup (bookmarks->tips, path);
 }
+
+
+void
+bookmarks_set_max_lines (Bookmarks   *bookmarks,
+			 int          max_lines)
+{
+	bookmarks->max_lines = max_lines;
+}
+
+
+int
+bookmarks_get_max_lines (Bookmarks   *bookmarks)
+{
+	return bookmarks->max_lines;
+}
