@@ -151,8 +151,9 @@ image_data_free (ImageData *idata)
 {
 	int i;
 
-	if (idata->filename)
-		g_free (idata->filename);
+	g_free (idata->comment);
+	g_free (idata->filename);
+
 	if (idata->thumb)
 		g_object_unref (G_OBJECT (idata->thumb));
 
@@ -1246,6 +1247,7 @@ get_sortfunc (CatalogPngExporter *ce)
 		break;
 	default:
 		func = comp_func_none;
+		break;
 	}
 
 	return func;
@@ -1290,7 +1292,7 @@ load_next_file (CatalogPngExporter *ce)
 	g_free (ce->info);
 
 	utf8_name = g_locale_to_utf8 (file_name_from_path (filename), -1, 0, 0, 0);
-	ce->info = g_strdup_printf (_("Loading image : %s"), utf8_name);
+	ce->info = g_strdup_printf (_("Loading image: %s"), utf8_name);
 	g_free (utf8_name);
 
 	g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[INFO], 
@@ -1482,7 +1484,7 @@ begin_page (CatalogPngExporter *ce,
 
 	filename = _g_get_name_from_template (ce->templatev, ce->start_at + page_n - 1);
 	utf8_name = g_locale_to_utf8 (filename, -1, 0, 0, 0);
-	ce->info = g_strdup_printf (_("Creating image : %s.%s"), 
+	ce->info = g_strdup_printf (_("Creating image: %s.%s"), 
 				    utf8_name, 
 				    ce->file_type);
 	g_free (utf8_name);
@@ -1505,7 +1507,7 @@ begin_page (CatalogPngExporter *ce,
 	g_free (path);
 
 	if (uri == NULL) {
-		g_warning ("URI not valid : %s", path);
+		g_warning ("URI not valid: %s", path);
 		return;
 	}
 
@@ -1775,12 +1777,11 @@ static void
 paint_comment (CatalogPngExporter *ce,
 	       int                 x,
 	       int                 y,
-	       char               *text,
+	       char               *utf8_text,
 	       int                *height)
 {
 	PangoRectangle        bounds;
 	const char           *font_name;
-	char                 *utf8_text;
 	char                 *escaped_text;
 	char                 *marked_text;
 	char                 *parsed_text;
@@ -1801,11 +1802,9 @@ paint_comment (CatalogPngExporter *ce,
 		pango_attr_list_ref (original_attr_list);
 	x += FRAME_BORDER;
 
-	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
 	escaped_text = g_markup_escape_text (utf8_text, -1);
 	marked_text = g_strdup_printf ("<i>%s</i>", escaped_text);
 	g_free (escaped_text);
-	g_free (utf8_text);
 
 	if (! pango_parse_markup (marked_text, -1,
 				  0,
@@ -1813,7 +1812,7 @@ paint_comment (CatalogPngExporter *ce,
 				  &parsed_text, 
 				  NULL,
 				  &error)) {
-		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error : %s",  error->message, text);
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, utf8_text);
 		g_error_free (error);
 		g_free (marked_text);
 		pango_attr_list_unref (original_attr_list);
