@@ -34,6 +34,7 @@
 #include "file-data.h"
 #include "file-list.h"
 #include "file-utils.h"
+#include "gthumb-marshal.h"
 #include "image-list.h"
 #include "thumb-loader.h"
 #include "typedefs.h"
@@ -44,11 +45,20 @@
 #define COL_SPACING    14
 #define ADD_LIST_DELAY 75
 #define ADD_LIST_CHUNK 333
+#define SCROLL_DELAY   20
 
-static GdkPixbuf *unknown_pixbuf = NULL;
+enum {
+	BUSY,
+	IDLE,
+	LAST_SIGNAL
+};
+
+static GObjectClass *parent_class = NULL;
+static guint         gth_file_list_signals[LAST_SIGNAL] = { 0 };
+static GdkPixbuf    *unknown_pixbuf = NULL;
 
 
-static gint
+static int
 comp_func_name (gconstpointer  ptr1,
 		gconstpointer  ptr2)
 {
@@ -65,7 +75,7 @@ comp_func_name (gconstpointer  ptr1,
 }
 
 
-static gint
+static int
 comp_func_size (gconstpointer  ptr1,
 		gconstpointer  ptr2)
 {
@@ -86,7 +96,7 @@ comp_func_size (gconstpointer  ptr1,
 }
 
 
-static gint
+static int
 comp_func_time (gconstpointer  ptr1,
 		gconstpointer  ptr2)
 {
@@ -107,7 +117,7 @@ comp_func_time (gconstpointer  ptr1,
 }
 
 
-static gint
+static int
 comp_func_path (gconstpointer  ptr1,
 		gconstpointer  ptr2)
 {
@@ -124,7 +134,7 @@ comp_func_path (gconstpointer  ptr1,
 }
 
 
-static gint
+static int
 comp_func_none (gconstpointer  ptr1,
 		gconstpointer  ptr2)
 {
@@ -133,7 +143,7 @@ comp_func_none (gconstpointer  ptr1,
 
 
 static void
-set_unknown_icon (FileList *file_list, gint pos)
+set_unknown_icon (GthFileList *file_list, int pos)
 {
 	image_list_set_image_pixbuf (IMAGE_LIST (file_list->ilist), 
 				     pos, unknown_pixbuf);
@@ -141,7 +151,7 @@ set_unknown_icon (FileList *file_list, gint pos)
 
 
 static void 
-update_thumb_in_clist (FileList *file_list)
+update_thumb_in_clist (GthFileList *file_list)
 {
 	GdkPixbuf *pixbuf;
 
@@ -156,21 +166,27 @@ update_thumb_in_clist (FileList *file_list)
 }
 
 
-static void file_list_update_next_thumb (FileList *file_list);
-static void interrupt_thumbs__part2 (FileList *file_list);
+static void gth_file_list_update_next_thumb (GthFileList *file_list);
 
 
 static void 
 load_thumb_error_cb (ThumbLoader *tl,
 		     gpointer data)
 {
-	FileList *file_list = data;
+	GthFileList *file_list = data;
+	float        percent;
 
 	set_unknown_icon (file_list, file_list->thumb_pos);
 	file_list->thumb_fd->error = TRUE;
 	file_list->thumb_fd->thumb = FALSE;
 
-	file_list_update_next_thumb (file_list);
+	percent = ((float) (file_list->thumbs_num - 1) 
+		   / IMAGE_LIST (file_list->ilist)->images);
+
+	if (file_list->progress_func) 
+		file_list->progress_func (percent, file_list->progress_data);
+
+	gth_file_list_update_next_thumb (file_list);
 }
 
 
@@ -178,19 +194,20 @@ static void
 load_thumb_done_cb (ThumbLoader *tl,
 		    gpointer data)
 {
-	FileList *file_list = data;
-	gfloat    percent;
+	GthFileList *file_list = data;
+	float        percent;
 
 	update_thumb_in_clist (file_list);
 	file_list->thumb_fd->error = FALSE;
 	file_list->thumb_fd->thumb = TRUE;
 
-	percent = ((float) (file_list->thumbs_num) 
+	percent = ((float) (file_list->thumbs_num - 1) 
 		   / IMAGE_LIST (file_list->ilist)->images);
+
 	if (file_list->progress_func) 
 		file_list->progress_func (percent, file_list->progress_data);
 
-	file_list_update_next_thumb (file_list);
+	gth_file_list_update_next_thumb (file_list);
 }
 
 
@@ -217,25 +234,127 @@ get_compfunc_from_method (SortMethod sort_method)
 		break;
 	default:
 		func = comp_func_none;
+		break;
 	}
 
 	return func;
 }
 
 
-FileList*   
-file_list_new ()
+static void
+free_file_data_list (GList *list)
 {
-	FileList *file_list;
-	GtkWidget *ilist;
-	GtkAdjustment *adj;
-        GtkWidget *vsb, *hbox;
-	GtkWidget *frame; 
-	ImageListViewMode view_mode;
+	GList *scan;
+	for (scan = list; scan; scan = scan->next) 
+		file_data_unref (scan->data);
+	g_list_free (list);
+}
+
+
+static void
+gth_file_list_free_list (GthFileList *file_list)
+{
+	g_return_if_fail (file_list != NULL);
+
+	free_file_data_list (file_list->list);
+	file_list->list = NULL;
+}
+
+
+static void 
+gth_file_list_finalize (GObject *object)
+{
+	GthFileList *file_list;
+
+        g_return_if_fail (GTH_IS_FILE_LIST (object));
+	file_list = GTH_FILE_LIST (object);
+
+	gth_file_list_free_list (file_list);
+	g_object_unref (G_OBJECT (file_list->thumb_loader));
+	g_object_unref (G_OBJECT (unknown_pixbuf));
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
+static void
+gth_file_list_class_init (GthFileListClass *class)
+{
+	GObjectClass *object_class;
+
+	parent_class = g_type_class_peek_parent (class);
+
+	gth_file_list_signals[BUSY] =
+		g_signal_new ("busy",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GthFileListClass, busy),
+			      NULL, NULL,
+			      gthumb_marshal_VOID__VOID,
+			      G_TYPE_NONE, 
+			      0);
+	gth_file_list_signals[IDLE] =
+		g_signal_new ("idle",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GthFileListClass, idle),
+			      NULL, NULL,
+			      gthumb_marshal_VOID__VOID,
+			      G_TYPE_NONE, 
+			      0);
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = gth_file_list_finalize;
+}
+
+
+static gboolean
+update_thumbnails_cb (gpointer data)
+{
+	GthFileList *file_list = data;
+
+	if (file_list->scroll_timer != 0)
+		g_source_remove (file_list->scroll_timer);
+
+	gth_file_list_restart_thumbs (file_list, TRUE);
+
+	file_list->scroll_timer = 0;
+
+	return FALSE;
+}
+
+
+static gboolean
+file_list_adj_value_changed (GtkAdjustment *adj, 
+			     GthFileList   *file_list)
+{	
+	if (file_list->scroll_timer != 0) {
+		g_source_remove (file_list->scroll_timer);
+		file_list->scroll_timer = 0;
+	}
+
+	if (image_list_is_frozen (IMAGE_LIST (file_list->ilist)))
+		return FALSE;
+
+	file_list->scroll_timer = g_timeout_add (SCROLL_DELAY,
+						 update_thumbnails_cb,
+						 file_list);
+
+	return FALSE;
+}
+
+
+static void
+gth_file_list_init (GthFileList *file_list)
+{
+	GtkWidget         *ilist;
+	GtkAdjustment     *adj;
+        GtkWidget         *vsb, *hbox;
+	GtkWidget         *frame; 
+	ImageListViewMode  view_mode;
 
-	file_list = g_new (FileList, 1);
-	
 	/* set the default values. */
+
 	file_list->list              = NULL;
 	file_list->sort_method       = pref_get_arrange_type ();
 	file_list->sort_type         = pref_get_sort_order ();
@@ -251,6 +370,7 @@ file_list_new ()
 	file_list->progress_data     = NULL;
 	file_list->interrupt_done_func = NULL;
 	file_list->interrupt_done_data = NULL;
+	file_list->scroll_timer      = 0;
 
 	if (unknown_pixbuf == NULL)
 		unknown_pixbuf = gdk_pixbuf_new_from_inline (-1, unknown_48_rgba, FALSE, NULL);
@@ -269,10 +389,12 @@ file_list_new ()
 	/* Create the widgets. */
 
 	/* vertical scrollbar. */
+
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 1, 1, 1, 1));
         vsb = gtk_vscrollbar_new (adj);
 
 	/* image list. */
+
 	ilist = image_list_new (eel_gconf_get_integer (PREF_THUMBNAIL_SIZE) + THUMB_BORDER,
 				adj, 
 				0);
@@ -296,6 +418,7 @@ file_list_new ()
 	gtk_container_add (GTK_CONTAINER (frame), ilist);
 
 	/**/
+
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), vsb, FALSE, FALSE, 0);
@@ -303,57 +426,74 @@ file_list_new ()
 	file_list->ilist = ilist;
 	file_list->root_widget = hbox;
 
-	return file_list;
+	/* signals */
+
+	g_signal_connect_after (G_OBJECT (adj), 
+				"value_changed",
+				G_CALLBACK (file_list_adj_value_changed),
+				file_list);
+
+	/* FIXME */
+	g_signal_connect_after (G_OBJECT (adj), 
+				"changed",
+				G_CALLBACK (file_list_adj_value_changed), 
+				file_list);
 }
 
 
-static void
-free_file_data_list (GList *list)
+GType
+gth_file_list_get_type ()
 {
-	GList *scan;
-	for (scan = list; scan; scan = scan->next) 
-		file_data_unref (scan->data);
-	g_list_free (list);
+	static guint type = 0;
+
+	if (! type) {
+		GTypeInfo type_info = {
+			sizeof (GthFileListClass),
+                        NULL,
+                        NULL,
+                        (GClassInitFunc) gth_file_list_class_init,
+                        NULL,
+                        NULL,
+                        sizeof (GthFileList),
+                        0,
+                        (GInstanceInitFunc) gth_file_list_init
+                };
+
+                type = g_type_register_static (G_TYPE_OBJECT,
+                                               "GthFileList",
+                                               &type_info,
+                                               0);
+        }
+
+        return type;
 }
 
 
-static void
-file_list_free_list (FileList *file_list)
+GthFileList*   
+gth_file_list_new ()
 {
-	g_return_if_fail (file_list != NULL);
-
-	free_file_data_list (file_list->list);
-	file_list->list = NULL;
-}
-
-
-void
-file_list_free (FileList *file_list)
-{
-	file_list_free_list (file_list);
-	g_object_unref (G_OBJECT (file_list->thumb_loader));
-	g_free (file_list);
-	g_object_unref (G_OBJECT (unknown_pixbuf));
+	return GTH_FILE_LIST (g_object_new (GTH_TYPE_FILE_LIST, NULL));
 }
 
 
 
-/* ----- file_list_set_list help functions ----- */
+/* ----- gth_file_list_set_list help functions ----- */
 
 
 typedef struct {
-	FileList *file_list;
-	GList    *filtered;
-	GList    *uri_list;
-	DoneFunc  done_func;
-	gpointer  done_func_data;
-	guint     timeout_id;
-	gboolean  doing_thumbs;
+	GthFileList *file_list;
+	GList       *filtered;
+	GList       *uri_list;
+	DoneFunc     done_func;
+	gpointer     done_func_data;
+	guint        timeout_id;
+	gboolean     doing_thumbs;
+	gboolean     enable_thumbs;
 } GetFileInfoData;
 
 
 static GetFileInfoData *
-get_file_info_data_new (FileList *file_list, 
+get_file_info_data_new (GthFileList *file_list, 
 			DoneFunc  done_func,
 			gpointer  done_func_data)
 {
@@ -366,7 +506,8 @@ get_file_info_data_new (FileList *file_list,
 	data->done_func = done_func;
 	data->done_func_data = done_func_data;
 	data->timeout_id = 0;
-	data->doing_thumbs = FALSE;
+	data->doing_thumbs = file_list->doing_thumbs;
+	data->enable_thumbs = file_list->enable_thumbs;
 
 	return data;
 }
@@ -396,7 +537,7 @@ get_file_info_data_free (GetFileInfoData *data)
 }
 
 
-/* -- file_list_set_list -- */
+/* -- gth_file_list_set_list -- */
 
 
 static gboolean add_list_in_chunks (gpointer callback_data);
@@ -408,10 +549,12 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 				 gpointer callback_data)
 {
 	GetFileInfoData *gfi_data = callback_data;
-	FileList        *file_list = gfi_data->file_list;
+	GthFileList     *file_list = gfi_data->file_list;
 	GList           *scan;
 	DoneFunc         done_func;
 
+	g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[IDLE], 0);
+
 	if (file_list->interrupt_set_list) {
 		done_func = file_list->interrupt_done_func;
 		file_list->interrupt_done_func = NULL;
@@ -450,9 +593,11 @@ void
 set_list__step2 (GetFileInfoData *gfi_data)
 {
 	GnomeVFSAsyncHandle *handle;
-	FileList            *file_list = gfi_data->file_list;
+	GthFileList         *file_list = gfi_data->file_list;
 
 	if (file_list->interrupt_set_list) {
+		g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[IDLE], 0);
+
 		if (file_list->interrupt_done_func != NULL)
 			(*file_list->interrupt_done_func) (file_list->interrupt_done_data);
 		file_list->interrupt_done_func = NULL;
@@ -460,7 +605,7 @@ set_list__step2 (GetFileInfoData *gfi_data)
 		return;
 	}
 
- 	file_list_free_list (file_list);
+ 	gth_file_list_free_list (file_list);
 
 	gnome_vfs_async_get_file_info (&handle,
 				       gfi_data->uri_list,
@@ -473,21 +618,26 @@ set_list__step2 (GetFileInfoData *gfi_data)
 
 
 void 
-file_list_set_list (FileList *file_list,
-		    GList    *new_list,
-		    DoneFunc  done_func,
-		    gpointer  done_func_data)
+gth_file_list_set_list (GthFileList *file_list,
+			GList       *new_list,
+			DoneFunc     done_func,
+			gpointer     done_func_data)
 {
 	GetFileInfoData *gfi_data;
 	GList           *scan;
+	gboolean         fast_file_type;
 
 	g_return_if_fail (file_list != NULL);
 
+	g_signal_emit (G_OBJECT (file_list), gth_file_list_signals[BUSY], 0);
+
 	file_list->interrupt_set_list = FALSE;
 	gfi_data = get_file_info_data_new (file_list, 
 					   done_func, 
 					   done_func_data);
 
+	fast_file_type = eel_gconf_get_boolean (PREF_FAST_FILE_TYPE);
+
 	for (scan = new_list; scan; scan = scan->next) {
 		char        *full_path = scan->data;
 		const char  *name_only = file_name_from_path (full_path);
@@ -496,7 +646,7 @@ file_list_set_list (FileList *file_list,
 
 		if ((! gfi_data->file_list->show_dot_files 
 		     && file_is_hidden (name_only))
-		    || ! file_is_image (full_path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE)))
+		    || ! file_is_image (full_path, fast_file_type))
 			continue;
 
 		escaped = gnome_vfs_escape_path_string (full_path);
@@ -508,18 +658,18 @@ file_list_set_list (FileList *file_list,
 	}
 
 	if (file_list->doing_thumbs)
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) set_list__step2, 
-					    gfi_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) set_list__step2, 
+						gfi_data);
 	else
 		set_list__step2 (gfi_data);
 }
 
 
 void
-file_list_interrupt_set_list (FileList *file_list,
-			      DoneFunc done_func,
-			      gpointer done_data)
+gth_file_list_interrupt_set_list (GthFileList *file_list,
+				  DoneFunc     done_func,
+				  gpointer     done_data)
 {
 	g_return_if_fail (file_list != NULL);
 
@@ -529,14 +679,14 @@ file_list_interrupt_set_list (FileList *file_list,
 }
 
 
-/* -- file_list_add_list -- */
+/* -- gth_file_list_add_list -- */
 
 
 static gboolean
 add_list_in_chunks (gpointer callback_data)
 {
 	GetFileInfoData *gfi_data = callback_data;
-	FileList        *file_list = gfi_data->file_list;
+	GthFileList     *file_list = gfi_data->file_list;
 	GtkWidget       *ilist = file_list->ilist;
 	GList           *scan, *chunk;
 	DoneFunc         done_func;
@@ -548,11 +698,13 @@ add_list_in_chunks (gpointer callback_data)
 	}
 
 	if (file_list->interrupt_set_list) {
+		file_list->enable_thumbs = gfi_data->enable_thumbs;
+
 		done_func = file_list->interrupt_done_func;
 		file_list->interrupt_done_func = NULL;
-
 		if (done_func != NULL)
 			(*done_func) (file_list->interrupt_done_data);
+
 		get_file_info_data_free (gfi_data);
 
 		return FALSE;
@@ -561,9 +713,11 @@ add_list_in_chunks (gpointer callback_data)
 	if (gfi_data->filtered == NULL) {
 		image_list_sort (IMAGE_LIST (ilist));
 
+		file_list->enable_thumbs = gfi_data->enable_thumbs;
+
 		if ((file_list->list != NULL) && file_list->enable_thumbs) { 
 			file_list->doing_thumbs = TRUE;
-			file_list_update_next_thumb (file_list);
+			gth_file_list_update_next_thumb (file_list);
 		}
 
 		done_func = gfi_data->done_func;
@@ -576,7 +730,10 @@ add_list_in_chunks (gpointer callback_data)
 		
 		return FALSE;
 	}
-	
+
+	if (file_list->enable_thumbs)
+		file_list->enable_thumbs = FALSE;
+
 	/**/
 
 	image_list_freeze (IMAGE_LIST (ilist));
@@ -587,12 +744,12 @@ add_list_in_chunks (gpointer callback_data)
 		
 		pos = image_list_append (IMAGE_LIST (ilist),
 					 unknown_pixbuf,
-					 fd->name,
+					 fd->utf8_name,
 					 fd->comment);
 		
 		image_list_set_image_data (IMAGE_LIST (ilist), pos, fd);
 	}
-	
+
 	image_list_thaw (IMAGE_LIST (ilist));
 	
 	if ((scan != NULL) && (scan->prev != NULL)) {
@@ -608,7 +765,7 @@ add_list_in_chunks (gpointer callback_data)
 					      add_list_in_chunks, 
 					      gfi_data);
 	
-	return TRUE;
+	return FALSE;
 }
 
 
@@ -618,7 +775,7 @@ add_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 				 gpointer callback_data)
 {
 	GetFileInfoData *gfi_data = callback_data;
-	FileList        *file_list = gfi_data->file_list;
+	GthFileList        *file_list = gfi_data->file_list;
 	GList           *scan;
 	DoneFunc         done_func;
 
@@ -651,13 +808,6 @@ add_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 		gfi_data->filtered = g_list_prepend (gfi_data->filtered, fd);
 	}
 
-	if (! gfi_data->doing_thumbs)
-		for (scan = file_list->list; scan; scan = scan->next) {
-			FileData *fd = scan->data;
-			fd->thumb = TRUE;
-			fd->error = FALSE;
-		}
-
 	add_list_in_chunks (gfi_data);
 
 	return;
@@ -668,7 +818,7 @@ void
 add_list__step2 (GetFileInfoData *gfi_data)
 {
 	GnomeVFSAsyncHandle *handle;
-	FileList            *file_list = gfi_data->file_list;
+	GthFileList            *file_list = gfi_data->file_list;
 
 	if (file_list->interrupt_set_list) {
 		if (file_list->interrupt_done_func != NULL)
@@ -689,10 +839,10 @@ add_list__step2 (GetFileInfoData *gfi_data)
 
 
 void 
-file_list_add_list (FileList *file_list,
-		    GList    *new_list,
-		    DoneFunc  done_func,
-		    gpointer  done_func_data)
+gth_file_list_add_list (GthFileList *file_list,
+			GList       *new_list,
+			DoneFunc     done_func,
+			gpointer     done_func_data)
 {
 	GetFileInfoData *gfi_data;
 	GList           *scan;
@@ -710,7 +860,7 @@ file_list_add_list (FileList *file_list,
 		char        *escaped;
 		GnomeVFSURI *uri;
 
-		if (file_list_pos_from_path (file_list, full_path) != -1)
+		if (gth_file_list_pos_from_path (file_list, full_path) != -1)
 			continue;
 
 		if ((! gfi_data->file_list->show_dot_files 
@@ -733,12 +883,10 @@ file_list_add_list (FileList *file_list,
 		return;
 	}
 
-	gfi_data->doing_thumbs = file_list->doing_thumbs;
-
 	if (file_list->doing_thumbs)
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) add_list__step2, 
-					    gfi_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) add_list__step2, 
+						gfi_data);
 	else
 		add_list__step2 (gfi_data);
 }
@@ -748,7 +896,7 @@ file_list_add_list (FileList *file_list,
 
 
 typedef struct {
-	FileList *file_list;
+	GthFileList *file_list;
 	gboolean  restart_thumbs;
 	int       i_data;
 	char     *s_data;
@@ -756,7 +904,7 @@ typedef struct {
 
 
 static InterruptThumbsData *
-it_data_new (FileList   *file_list,
+it_data_new (GthFileList   *file_list,
 	     gboolean    restart_thumbs,
 	     int         i_data,
 	     const char *s_data)
@@ -791,7 +939,7 @@ it_data_free (InterruptThumbsData *it_data)
 void
 set_sort_method__step2 (InterruptThumbsData *it_data)
 {
-	FileList   *file_list = it_data->file_list;
+	GthFileList   *file_list = it_data->file_list;
 	SortMethod  new_method = it_data->i_data;
 
 	file_list->sort_method = new_method;
@@ -801,7 +949,7 @@ set_sort_method__step2 (InterruptThumbsData *it_data)
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
 	
 	it_data_free (it_data);
@@ -809,8 +957,8 @@ set_sort_method__step2 (InterruptThumbsData *it_data)
 
 
 void 
-file_list_set_sort_method (FileList   *file_list,
-			   SortMethod  new_method)
+gth_file_list_set_sort_method (GthFileList *file_list,
+			       SortMethod   new_method)
 {
 	InterruptThumbsData *it_data;
 
@@ -821,9 +969,9 @@ file_list_set_sort_method (FileList   *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, new_method, NULL);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) set_sort_method__step2,
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) set_sort_method__step2,
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, new_method, NULL);
 		set_sort_method__step2 (it_data);
@@ -837,7 +985,7 @@ file_list_set_sort_method (FileList   *file_list,
 void
 set_sort_type__step2 (InterruptThumbsData *it_data)
 {
-	FileList    *file_list = it_data->file_list;
+	GthFileList    *file_list = it_data->file_list;
 	GtkSortType  sort_type = it_data->i_data;
 
 	file_list->sort_type = sort_type;
@@ -846,7 +994,7 @@ set_sort_type__step2 (InterruptThumbsData *it_data)
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
 	
 	it_data_free (it_data);
@@ -854,8 +1002,8 @@ set_sort_type__step2 (InterruptThumbsData *it_data)
 
 
 void
-file_list_set_sort_type (FileList    *file_list,
-			 GtkSortType  sort_type)
+gth_file_list_set_sort_type (GthFileList *file_list,
+			     GtkSortType  sort_type)
 {
 	InterruptThumbsData *it_data;
 
@@ -863,9 +1011,9 @@ file_list_set_sort_type (FileList    *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, sort_type, NULL);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) set_sort_type__step2,
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) set_sort_type__step2,
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, sort_type, NULL);
 		set_sort_type__step2 (it_data);
@@ -876,15 +1024,15 @@ file_list_set_sort_type (FileList    *file_list,
 /* -- -- */
 
 
-static void file_list_thumb_cleanup (FileList *file_list);
+static void gth_file_list_thumb_cleanup (GthFileList *file_list);
 
 
 static void
-interrupt_thumbs__part2 (FileList *file_list)
+interrupt_thumbs__part2 (GthFileList *file_list)
 {
 	DoneFunc done_func = file_list->interrupt_done_func;
 
-	file_list_thumb_cleanup (file_list);
+	gth_file_list_thumb_cleanup (file_list);
 	file_list->interrupt_done_func = NULL;
 	if (done_func != NULL)
 		(*done_func) (file_list->interrupt_done_data);
@@ -892,9 +1040,9 @@ interrupt_thumbs__part2 (FileList *file_list)
 
 
 void 
-file_list_interrupt_thumbs (FileList *file_list, 
-			    DoneFunc  done_func,
-			    gpointer  done_func_data)
+gth_file_list_interrupt_thumbs (GthFileList *file_list, 
+				DoneFunc     done_func,
+				gpointer     done_func_data)
 {
 	g_return_if_fail (file_list != NULL);
 
@@ -908,9 +1056,9 @@ file_list_interrupt_thumbs (FileList *file_list,
 }
 
 
-gint 
-file_list_pos_from_path (FileList    *file_list, 
-			 const char  *path)
+int 
+gth_file_list_pos_from_path (GthFileList *file_list, 
+			     const char  *path)
 {
 	GList *scan;
 	int    i;
@@ -938,7 +1086,7 @@ file_list_pos_from_path (FileList    *file_list,
 
 
 GList *
-file_list_get_all (FileList *file_list)
+gth_file_list_get_all (GthFileList *file_list)
 {
 	GList *list;
 	GList *scan;
@@ -955,8 +1103,8 @@ file_list_get_all (FileList *file_list)
 }
 
 
-gint 
-file_list_get_length (FileList *file_list)
+int 
+gth_file_list_get_length (GthFileList *file_list)
 {
 	g_return_val_if_fail (file_list != NULL, 0);
 	return g_list_length (file_list->list);
@@ -964,7 +1112,7 @@ file_list_get_length (FileList *file_list)
 
 
 GList *
-file_list_get_selection (FileList *file_list)
+gth_file_list_get_selection (GthFileList *file_list)
 {
 	GList *list = NULL;
 	GList *scan;
@@ -982,7 +1130,7 @@ file_list_get_selection (FileList *file_list)
 
 
 GList *
-file_list_get_selection_as_fd (FileList *file_list)
+gth_file_list_get_selection_as_fd (GthFileList *file_list)
 {
 	GList *list = NULL;
 	GList *scan;
@@ -999,8 +1147,8 @@ file_list_get_selection_as_fd (FileList *file_list)
 }
 
 
-gint 
-file_list_get_selection_length (FileList *file_list)
+int 
+gth_file_list_get_selection_length (GthFileList *file_list)
 {
 	g_return_val_if_fail (file_list != NULL, 0);
 	return g_list_length (IMAGE_LIST (file_list->ilist)->selection);
@@ -1008,7 +1156,7 @@ file_list_get_selection_length (FileList *file_list)
 
 
 gchar *
-file_list_path_from_pos (FileList *file_list,
+gth_file_list_path_from_pos (GthFileList *file_list,
 			 int       pos)
 {
 	FileData *fd;
@@ -1027,8 +1175,8 @@ file_list_path_from_pos (FileList *file_list,
 
 
 gboolean
-file_list_is_selected (FileList *file_list,
-		       int       pos)
+gth_file_list_is_selected (GthFileList *file_list,
+			   int          pos)
 {
 	GList *scan;
 
@@ -1044,8 +1192,8 @@ file_list_is_selected (FileList *file_list,
 
 
 void 
-file_list_select_image_by_pos (FileList *file_list,
-			       int       pos)
+gth_file_list_select_image_by_pos (GthFileList *file_list,
+				   int          pos)
 {
 	GtkWidget        *ilist;
 	GthumbVisibility  visibility;
@@ -1060,7 +1208,7 @@ file_list_select_image_by_pos (FileList *file_list,
 
 	visibility = image_list_image_is_visible (IMAGE_LIST (ilist), pos);
 	if (visibility != GTHUMB_VISIBILITY_FULL) {
-		gdouble offset;
+		double offset;
 		switch (visibility) {
 		case GTHUMB_VISIBILITY_NONE:
 			offset = 0.5; 
@@ -1081,7 +1229,7 @@ file_list_select_image_by_pos (FileList *file_list,
 
 
 void 
-file_list_select_all (FileList *file_list)
+gth_file_list_select_all (GthFileList *file_list)
 {
 	g_return_if_fail (file_list != NULL);
 	image_list_select_all (IMAGE_LIST (file_list->ilist));
@@ -1089,7 +1237,7 @@ file_list_select_all (FileList *file_list)
 
 
 void 
-file_list_unselect_all (FileList *file_list)
+gth_file_list_unselect_all (GthFileList *file_list)
 {
 	g_return_if_fail (file_list != NULL);
 	image_list_unselect_all (IMAGE_LIST (file_list->ilist), NULL, FALSE);
@@ -1097,7 +1245,7 @@ file_list_unselect_all (FileList *file_list)
 
 
 static void
-enable_thumbs__step2 (FileList *file_list)
+enable_thumbs__step2 (GthFileList *file_list)
 {
 	ImageList *ilist = IMAGE_LIST (file_list->ilist);
 	int        i;
@@ -1107,28 +1255,28 @@ enable_thumbs__step2 (FileList *file_list)
 }
 
 
-static void file_list_update_thumbs (FileList *file_list);
+static void gth_file_list_update_thumbs (GthFileList *file_list);
 
 
 void
-file_list_enable_thumbs (FileList *file_list,
-			 gboolean  enable)
+gth_file_list_enable_thumbs (GthFileList *file_list,
+			     gboolean     enable)
 {
 	g_return_if_fail (file_list != NULL);
 
 	file_list->enable_thumbs = enable;
 
 	if (file_list->enable_thumbs)
-		file_list_update_thumbs (file_list);
+		gth_file_list_update_thumbs (file_list);
 	else 
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) enable_thumbs__step2, 
-					    file_list);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) enable_thumbs__step2, 
+						file_list);
 }
 
 
 void
-file_list_set_progress_func (FileList *file_list,
+gth_file_list_set_progress_func (GthFileList *file_list,
 			    ProgressFunc func,
 			    gpointer data)
 {
@@ -1140,7 +1288,7 @@ file_list_set_progress_func (FileList *file_list,
 
 
 /**
- * file_list_next_image:
+ * gth_file_list_next_image:
  * @file_list: 
  * @pos: 
  * @without_error: 
@@ -1152,10 +1300,10 @@ file_list_set_progress_func (FileList *file_list,
  * Return value: the next row that contains an image or -1 if no row could
  * be found.
  **/
-gint
-file_list_next_image (FileList *file_list,
-		      int       pos,
-		      gboolean  without_error)
+int
+gth_file_list_next_image (GthFileList *file_list,
+			  int          pos,
+			  gboolean     without_error)
 {
 	int n;
 
@@ -1187,10 +1335,10 @@ file_list_next_image (FileList *file_list,
 }
 
 
-gint
-file_list_prev_image (FileList *file_list,
-		      int       pos,
-		      gboolean  without_error)
+int
+gth_file_list_prev_image (GthFileList *file_list,
+			  int          pos,
+			  gboolean     without_error)
 {
 	g_return_val_if_fail (file_list != NULL, -1);
 
@@ -1224,7 +1372,7 @@ file_list_prev_image (FileList *file_list,
 void
 delete_pos__step2 (InterruptThumbsData *it_data)
 {
-	FileList  *file_list = it_data->file_list;
+	GthFileList  *file_list = it_data->file_list;
 	int        pos = it_data->i_data;
 	ImageList *ilist = IMAGE_LIST (file_list->ilist);
 	FileData  *fd;
@@ -1240,7 +1388,7 @@ delete_pos__step2 (InterruptThumbsData *it_data)
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
 
 	it_data_free (it_data);
@@ -1248,8 +1396,8 @@ delete_pos__step2 (InterruptThumbsData *it_data)
 
 
 void
-file_list_delete_pos (FileList *file_list,
-		      int       pos)
+gth_file_list_delete_pos (GthFileList *file_list,
+			  int          pos)
 {
 	ImageList           *ilist;
 	InterruptThumbsData *it_data;
@@ -1262,9 +1410,9 @@ file_list_delete_pos (FileList *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, pos, NULL);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) delete_pos__step2, 
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) delete_pos__step2, 
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, pos, NULL);
 		delete_pos__step2 (it_data);
@@ -1278,7 +1426,7 @@ file_list_delete_pos (FileList *file_list,
 void
 rename_pos__step2 (InterruptThumbsData *it_data)
 {
-	FileList    *file_list = it_data->file_list;
+	GthFileList    *file_list = it_data->file_list;
 	int          pos = it_data->i_data;
 	char *       path = it_data->s_data;
 	ImageList   *ilist = IMAGE_LIST (file_list->ilist);
@@ -1289,12 +1437,12 @@ rename_pos__step2 (InterruptThumbsData *it_data)
 	file_data_set_path (fd, path);
 
 	/* Set the new name. */
-	image_list_set_image_text (ilist, pos, fd->name);
+	image_list_set_image_text (ilist, pos, fd->utf8_name);
 	image_list_sort (ilist); 
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
 
 	it_data_free (it_data);
@@ -1302,9 +1450,9 @@ rename_pos__step2 (InterruptThumbsData *it_data)
 
 
 void
-file_list_rename_pos (FileList   *file_list,
-		      int         pos, 
-		      const char *path)
+gth_file_list_rename_pos (GthFileList *file_list,
+			  int          pos, 
+			  const char  *path)
 {
 	ImageList           *ilist;
 	InterruptThumbsData *it_data;
@@ -1317,9 +1465,9 @@ file_list_rename_pos (FileList   *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, pos, path);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) rename_pos__step2, 
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) rename_pos__step2, 
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, pos, path);
 		rename_pos__step2 (it_data);
@@ -1333,7 +1481,7 @@ file_list_rename_pos (FileList   *file_list,
 void
 update_comment__step2 (InterruptThumbsData *it_data)
 {
-	FileList    *file_list = it_data->file_list;
+	GthFileList    *file_list = it_data->file_list;
 	int          pos = it_data->i_data;
 	ImageList   *ilist = IMAGE_LIST (file_list->ilist);
 	FileData    *fd;
@@ -1349,7 +1497,7 @@ update_comment__step2 (InterruptThumbsData *it_data)
 
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
 
 	it_data_free (it_data);
@@ -1357,8 +1505,8 @@ update_comment__step2 (InterruptThumbsData *it_data)
 
 
 void
-file_list_update_comment (FileList *file_list,
-			  int       pos)
+gth_file_list_update_comment (GthFileList *file_list,
+			      int          pos)
 {
 	ImageList           *ilist;
 	InterruptThumbsData *it_data;
@@ -1372,9 +1520,9 @@ file_list_update_comment (FileList *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, pos, NULL);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) update_comment__step2, 
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) update_comment__step2, 
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, pos, NULL);
 		update_comment__step2 (it_data);
@@ -1388,7 +1536,7 @@ file_list_update_comment (FileList *file_list,
 void
 set_thumbs_size__step2 (InterruptThumbsData *it_data)
 {
-	FileList    *file_list = it_data->file_list;
+	GthFileList    *file_list = it_data->file_list;
 	int          size = it_data->i_data;
 
 	file_list->thumb_size = size;
@@ -1407,18 +1555,20 @@ set_thumbs_size__step2 (InterruptThumbsData *it_data)
 	image_list_set_image_width (IMAGE_LIST (file_list->ilist), 
 				    size + THUMB_BORDER);
 
+	/* FIXME
 	if (it_data->restart_thumbs) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	}
+	*/
 
 	it_data_free (it_data);
 }
 
 
 void
-file_list_set_thumbs_size (FileList *file_list,
-			   int       size)
+gth_file_list_set_thumbs_size (GthFileList *file_list,
+			       int          size)
 {
 	InterruptThumbsData *it_data;
 
@@ -1427,9 +1577,9 @@ file_list_set_thumbs_size (FileList *file_list,
 
 	if (file_list->doing_thumbs) {
 		it_data = it_data_new (file_list, TRUE, size, NULL);
-		file_list_interrupt_thumbs (file_list, 
-					    (DoneFunc) set_thumbs_size__step2, 
-					    it_data);
+		gth_file_list_interrupt_thumbs (file_list, 
+						(DoneFunc) set_thumbs_size__step2, 
+						it_data);
 	} else {
 		it_data = it_data_new (file_list, FALSE, size, NULL);
 		set_thumbs_size__step2 (it_data);
@@ -1441,7 +1591,7 @@ file_list_set_thumbs_size (FileList *file_list,
 
 
 static void
-file_list_update_current_thumb (FileList *file_list)
+gth_file_list_update_current_thumb (GthFileList *file_list)
 {
 	if (path_is_file (file_list->thumb_fd->path)) {
 		thumb_loader_set_path (file_list->thumb_loader, 
@@ -1457,8 +1607,8 @@ file_list_update_current_thumb (FileList *file_list)
 
 
 void
-file_list_update_thumb (FileList     *file_list,
-			int           pos)
+gth_file_list_update_thumb (GthFileList  *file_list,
+			    int           pos)
 {
 	FileData  *fd;
 
@@ -1473,13 +1623,13 @@ file_list_update_thumb (FileList     *file_list,
 	file_list->thumb_pos = pos;
 	file_list->thumb_fd = fd;
 	
-	file_list_update_current_thumb (file_list);
+	gth_file_list_update_current_thumb (file_list);
 }
 
 
 void
-file_list_update_thumb_list (FileList     *file_list,
-			     GList        *list)
+gth_file_list_update_thumb_list (GthFileList  *file_list,
+				 GList        *list)
 {
 	GList     *scan;
 	ImageList *ilist;
@@ -1487,19 +1637,13 @@ file_list_update_thumb_list (FileList     *file_list,
 	if (! file_list->enable_thumbs)
 		return;
 
-	for (scan = file_list->list; scan; scan = scan->next) {
-		FileData *fd = scan->data;
-		fd->thumb = TRUE;
-		fd->error = FALSE;
-	}
-
 	ilist = IMAGE_LIST (file_list->ilist);
 	for (scan = list; scan; scan = scan->next) {
 		char      *path = scan->data;
 		FileData  *fd;
 		int        pos;
 
-		pos = file_list_pos_from_path (file_list, path);
+		pos = gth_file_list_pos_from_path (file_list, path);
 
 		if (pos == -1)
 			continue;
@@ -1512,7 +1656,7 @@ file_list_update_thumb_list (FileList     *file_list,
 	}
 	
 	file_list->doing_thumbs = TRUE;
-	file_list_update_next_thumb (file_list);
+	gth_file_list_update_next_thumb (file_list);
 }
 
 
@@ -1521,7 +1665,7 @@ file_list_update_thumb_list (FileList     *file_list,
 
 
 static void 
-file_list_thumb_cleanup (FileList *file_list)
+gth_file_list_thumb_cleanup (GthFileList *file_list)
 {
 	file_list->thumbs_num = 0;
 	file_list->doing_thumbs = FALSE;
@@ -1532,12 +1676,14 @@ file_list_thumb_cleanup (FileList *file_list)
 
 
 static void
-file_list_update_next_thumb (FileList *file_list)
+gth_file_list_update_next_thumb (GthFileList *file_list)
 {
 	ImageList *ilist;
 	FileData  *fd = NULL;
-	int        new_pos = -1;
+	int        first_pos, last_pos;
 	int        pos = -1;
+	GList     *scan;
+	int        new_pos = -1;
 
 	if (! file_list->doing_thumbs) {
 		interrupt_thumbs__part2 (file_list);
@@ -1547,64 +1693,49 @@ file_list_update_next_thumb (FileList *file_list)
 	ilist = IMAGE_LIST (file_list->ilist);
 
 	/* Find first visible undone. */
-	pos = image_list_get_first_visible (ilist);
-	if (pos != -1) { 
-		GList *scan; 
-		int n = ilist->images;
 
-		scan = image_list_get_list (ilist);
-		scan = g_list_nth (scan, pos);
-		while ((pos < n) && image_list_image_is_visible (ilist, pos)) {
-			Image *image = scan->data;			
-			fd = image->data;
-			if (! fd->thumb && ! fd->error) {
-				new_pos = pos;
-				break;
-			} else {
-				pos++;
-				scan = scan->next;
-			}
+	first_pos = image_list_get_first_visible (ilist);
+	last_pos = image_list_get_last_visible (ilist);
+
+	pos = first_pos;
+
+	if ((pos == -1) || (last_pos < first_pos)) {
+		gth_file_list_thumb_cleanup (file_list);
+		return;
+	}
+
+	scan = image_list_get_list (ilist);
+	scan = g_list_nth (scan, pos);
+	
+	while (pos <= last_pos) {
+		Image *image = scan->data;			
+		fd = image->data;
+		if (! fd->thumb && ! fd->error) {
+			new_pos = pos;
+			break;
+		} else {
+			pos++;
+			scan = scan->next;
 		}
 	}
 
-	/* If not found then search the first undone. */
 	if (new_pos == -1) {
-		GList *scan;
-		int    n = ilist->images;
-
-		pos = 0;
-		scan = image_list_get_list (ilist);
-		while ((pos < n) && (new_pos == -1)) {
-			Image *image = scan->data;			
-			fd = image->data;			
-			if (! fd->thumb && ! fd->error) 
-				new_pos = pos;
-			else {
-				pos++;
-				scan = scan->next;
-			}
-		}
-		
-		if (pos == n) {
-			/* Thumbnails creation terminated. */
-			file_list_thumb_cleanup (file_list);
-			return;
-		}
+		gth_file_list_thumb_cleanup (file_list);
+		return;
 	}
 
-	g_assert (new_pos != -1);
 	g_assert (fd != NULL);
 
 	file_list->thumb_pos = new_pos;
 	file_list->thumbs_num++;
 	file_list->thumb_fd = fd;
 
-	file_list_update_current_thumb (file_list);
+	gth_file_list_update_current_thumb (file_list);
 }
 
 
 static void 
-file_list_update_thumbs (FileList *file_list)
+gth_file_list_update_thumbs (GthFileList *file_list)
 {
 	GList *scan;
 
@@ -1620,20 +1751,20 @@ file_list_update_thumbs (FileList *file_list)
 	}
 		
 	file_list->doing_thumbs = TRUE;
-	file_list_update_next_thumb (file_list);
+	gth_file_list_update_next_thumb (file_list);
 }
 
 
 void 
-file_list_restart_thumbs (FileList *file_list,
-			  gboolean  _continue)
+gth_file_list_restart_thumbs (GthFileList *file_list,
+			      gboolean    _continue)
 {
 	if (! file_list->enable_thumbs)
 		return;
 
 	if (_continue) {
 		file_list->doing_thumbs = TRUE;
-		file_list_update_next_thumb (file_list);
+		gth_file_list_update_next_thumb (file_list);
 	} else
-		file_list_update_thumbs (file_list);	
+		gth_file_list_update_thumbs (file_list);	
 }
