@@ -174,18 +174,14 @@ iti_stop_editing (GThumbTextItem *iti)
 
 static void
 update_text (GThumbTextItem  *iti, 
-	     const char      *new_text)
+	     const char      *utf8_text)
 {
 	GThumbTextItemPrivate *priv = iti->_priv;
 	PangoAttrList         *attr_list = NULL;
         GError                *error = NULL;
 	char                  *marked_text;
 	char                  *parsed_text;
-	char                  *utf8_text;
 
-	g_return_if_fail (new_text != NULL);
-
-	utf8_text = g_locale_to_utf8 (new_text, -1, NULL, NULL, NULL);
 	g_return_if_fail (utf8_text != NULL);
 
 	if (priv->markup_str != NULL) {
@@ -195,7 +191,15 @@ update_text (GThumbTextItem  *iti,
 	} else
 		marked_text = g_markup_escape_text (utf8_text, -1);
 
-	g_free (utf8_text);
+	if (iti->text != NULL) {
+		g_free (iti->text);
+		iti->text = NULL;
+	}
+
+	if (priv->parsed_text != NULL) {
+		g_free (priv->parsed_text);
+		priv->parsed_text = NULL;
+	}
 
 	if (! pango_parse_markup (marked_text, -1,
 				  0,
@@ -203,19 +207,15 @@ update_text (GThumbTextItem  *iti,
 				  &parsed_text, 
 				  NULL,
 				  &error)) {
-		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error : %s",  error->message, new_text);
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, utf8_text);
 		g_error_free (error);
 		g_free (marked_text);
 		return;
 	}
+
 	g_free (marked_text);
 
-	if (iti->text != NULL)
-		g_free (iti->text);
-	iti->text = g_strdup (new_text);
-
-	if (priv->parsed_text != NULL)
-		g_free (priv->parsed_text);
+	iti->text = g_strdup (utf8_text);
 	priv->parsed_text = parsed_text;
 
 	pango_layout_set_attributes (priv->layout, attr_list);
@@ -270,7 +270,7 @@ iti_ensure_focus (GnomeCanvasItem *item)
 static void
 iti_entry_activate (GtkObject *widget, gpointer data)
 {
-	GThumbTextItem *item = GTHUMB_TEXT_ITEM (data);
+	GThumbTextItem        *item = data;
 	GThumbTextItemPrivate *priv=item->_priv;
 
 	gtk_widget_hide (priv->entry_win);
@@ -281,7 +281,6 @@ iti_entry_activate (GtkObject *widget, gpointer data)
 static void
 gthumb_text_item_realize (GnomeCanvasItem *item)
 {
-  /*GThumbTextItem *iti = GTHUMB_TEXT_ITEM (item);*/
 	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, realize, (item));
 }
 
@@ -289,7 +288,6 @@ gthumb_text_item_realize (GnomeCanvasItem *item)
 static void
 gthumb_text_item_unrealize (GnomeCanvasItem *item)
 {
-  /*	GThumbTextItem *iti = GTHUMB_TEXT_ITEM (item);*/
 	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, unrealize, (item));
 }
 
@@ -373,10 +371,10 @@ gthumb_text_item_update (GnomeCanvasItem *item,
 			 ArtSVP *clip_path, 
 			 int flags)
 {
-	GThumbTextItem *iti;
+	GThumbTextItem        *iti;
 	GThumbTextItemPrivate *priv;
 
-	iti = GTHUMB_TEXT_ITEM (item);
+	iti = (GThumbTextItem*) item;
 	priv = iti->_priv;
  
 	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, update, (item, affine, clip_path, flags));
@@ -420,11 +418,11 @@ gthumb_text_item_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	GThumbTextItemPrivate *priv;
 	GtkStateType state;
 
-	widget = GTK_WIDGET (item->canvas);
-	iti = GTHUMB_TEXT_ITEM (item);
-	priv = iti->_priv;
+	iti    = (GThumbTextItem*) item;
+	priv   = iti->_priv;
 
-	style = GTK_WIDGET (GNOME_CANVAS_ITEM (iti)->canvas)->style;
+	widget = (GtkWidget*) (item->canvas);
+	style = widget->style;
 
 	gc = style->fg_gc [GTK_STATE_NORMAL];
 	bgc = style->bg_gc [GTK_STATE_NORMAL];
@@ -438,7 +436,6 @@ gthumb_text_item_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	text_xofs = xofs - (iti->width - priv->layout_width - 2 * MARGIN_X) / 2;
 	text_yofs = yofs + MARGIN_Y;
 
-
 	if (GTK_WIDGET_HAS_FOCUS (widget))
 		state = GTK_STATE_SELECTED;
 	else
@@ -523,6 +520,7 @@ draw_pixbuf_aa (GdkPixbuf *pixbuf,
 	affine[5] -= y_offset;
 }
 
+
 static void
 gthumb_text_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buffer)
 {
@@ -547,8 +545,8 @@ gthumb_text_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buffer)
 
 	/* use a common routine to draw the label into the pixmap */
 	gthumb_text_item_draw (item, pixmap,
-				   ROUND (item->x1), ROUND (item->y1),
-				   width, height);
+			       ROUND (item->x1), ROUND (item->y1),
+			       width, height);
 
 	/* turn it into a pixbuf */
 	text_pixbuf = gdk_pixbuf_get_from_drawable
@@ -571,11 +569,11 @@ gthumb_text_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buffer)
 static void
 gthumb_text_item_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
 {
-	GThumbTextItem *iti;
+	GThumbTextItem        *iti;
 	GThumbTextItemPrivate *priv;
-	int width, height;
+	int                    width, height;
 
-	iti = GTHUMB_TEXT_ITEM (item);
+	iti = (GThumbTextItem*) item;
 	priv = iti->_priv;
 
 	width = priv->layout_width + 2 * MARGIN_X;
@@ -620,7 +618,7 @@ gthumb_text_item_event (GnomeCanvasItem *item, GdkEvent *event)
 	GThumbTextItem *iti;
 	GThumbTextItemPrivate *priv;
 
-	iti = GTHUMB_TEXT_ITEM (item);
+	iti = (GThumbTextItem*) item;
 	priv = iti->_priv;
 
 	switch (event->type) {
@@ -646,8 +644,8 @@ gthumb_text_item_event (GnomeCanvasItem *item, GdkEvent *event)
 static void
 gthumb_text_item_destroy (GtkObject *object)
 {
-	GThumbTextItem  *iti = GTHUMB_TEXT_ITEM (object);
-	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (object);
+	GThumbTextItem  *iti  = (GThumbTextItem*) object;
+	GnomeCanvasItem *item = (GnomeCanvasItem*) object;
 
 	gnome_canvas_request_redraw (item->canvas,
 				     ROUND (item->x1),
@@ -1006,11 +1004,10 @@ gthumb_text_item_get_text (GThumbTextItem *iti)
 
 	priv = iti->_priv;
 
-	if (iti->editing) {
+	if (iti->editing) 
 		return gtk_entry_get_text (GTK_ENTRY (priv->entry));
-	} else {
+	else 
 		return iti->text;
-	}
 }
 
 
