@@ -176,7 +176,7 @@ render_frame (GdkDrawable    *drawable,
 
 
 static char *
-escape_text (const char *text) 
+escape_locale_text (const char *text) 
 {
 	char *utf8_text;
 	char *escaped_text;
@@ -223,7 +223,7 @@ get_file_info (GThumbWindow *window)
 	time_t      timer;
 	struct tm  *tm;
 
-	e_filename = escape_text (file_name_from_path (window->image_path));
+	e_filename = escape_locale_text (file_name_from_path (window->image_path));
 
 	width = image_viewer_get_image_width (IMAGE_VIEWER (window->viewer));
 	height = image_viewer_get_image_height (IMAGE_VIEWER (window->viewer));
@@ -294,7 +294,7 @@ show_comment_on_image (GThumbWindow *window,
 	if (comment == NULL)
 		marked_text = g_strdup_printf ("<i>%s</i>", file_info);
 	else {
-		e_comment = escape_text (comment);
+		e_comment = g_markup_escape_text (comment, -1);
 		marked_text = g_strdup_printf ("<b>%s</b>\n<i>%s</i>", 
 					       e_comment, 
 					       file_info);
@@ -316,7 +316,7 @@ show_comment_on_image (GThumbWindow *window,
 				  &parsed_text, 
 				  NULL,
 				  &error)) {
-		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error : %s",  error->message, marked_text);
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, marked_text);
 		g_error_free (error);
 		g_free (marked_text);
 		g_object_unref (layout);
@@ -541,7 +541,7 @@ image_key_press_cb (GtkWidget *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		image_edit_comment_command_impl (NULL, window, NULL);
+		edit_current_edit_comment_command_impl (NULL, window, NULL);
 		break;
 
 		/* Edit categories. */
@@ -556,7 +556,7 @@ image_key_press_cb (GtkWidget *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		image_edit_categories_command_impl (NULL, window, NULL);
+		edit_current_edit_categories_command_impl (NULL, window, NULL);
 		break;
 
 		/* Flip image. */
